@@ -20,9 +20,11 @@
 #include <atomic>
 #include <cstddef>
 #include <cstdint>
+#include <functional>
 #include <memory>
 #include <string>
 #include <string_view>
+#include <utility>
 #include <variant>
 
 #include <fb303/ExportType.h>
@@ -39,8 +41,7 @@
 #include <folly/hash/Hash.h>
 #include <folly/synchronization/CallOnce.h>
 
-namespace facebook {
-namespace fb303 {
+namespace facebook::fb303 {
 
 /**
  * ThreadCachedServiceData wraps a ServiceData object, and exposes the same
@@ -317,8 +318,8 @@ class ThreadCachedServiceData {
     getServiceData()->getOptions(_return);
   }
 
-  typedef ServiceData::DynamicOptionGetter DynamicOptionGetter;
-  typedef ServiceData::DynamicOptionSetter DynamicOptionSetter;
+  using DynamicOptionGetter = ServiceData::DynamicOptionGetter;
+  using DynamicOptionSetter = ServiceData::DynamicOptionSetter;
 
   void registerDynamicOption(
       folly::StringPiece name,
@@ -328,9 +329,9 @@ class ThreadCachedServiceData {
         name, std::move(getter), std::move(setter));
   }
 
-  typedef ThreadLocalStatsMap::TLCounter TLCounter;
-  typedef ThreadLocalStatsMap::TLHistogram TLHistogram;
-  typedef ThreadLocalStatsMap::TLTimeseries TLTimeseries;
+  using TLCounter = ThreadLocalStatsMap::TLCounter;
+  using TLHistogram = ThreadLocalStatsMap::TLHistogram;
+  using TLTimeseries = ThreadLocalStatsMap::TLTimeseries;
 
   /**
    * Get a pointer to the ThreadLocalStatsMap object that caches stats for this
@@ -441,8 +442,28 @@ struct HistogramSpec {
   }
 };
 
-template <size_t N>
+namespace detail {
+struct Nothing {};
+
+template <typename CachedFieldType>
+struct CachedStorage {
+  const std::string* key;
+  CachedFieldType cached;
+};
+
+template <>
+struct CachedStorage<detail::Nothing> {
+  const std::string* key;
+  FOLLY_ATTR_NO_UNIQUE_ADDRESS detail::Nothing cached;
+};
+
+} // namespace detail
+
+template <size_t N, typename CachedType = detail::Nothing>
 class FormattedKeyHolder {
+ private:
+  using ValueType = detail::CachedStorage<CachedType>;
+
  public:
   using Subkey = std::variant<int64_t, std::string>;
   using SubkeyArray = std::array<Subkey, N>;
@@ -558,7 +579,7 @@ class FormattedKeyHolder {
   //  ThreadCachedServiceData::addStatValue() only accepts string references)
   using LocalMap = folly::F14FastMap<
       std::reference_wrapper<const SubkeyArray>,
-      const std::string*,
+      ValueType,
       SubkeyArrayHash,
       SubkeyArrayEqualTo>;
 
@@ -584,19 +605,15 @@ class FormattedKeyHolder {
    * and registers the stats export types if not registered already.
    */
   template <typename... Args>
-  const std::string& getFormattedKey(Args&&... subkeys) {
-    static_assert(sizeof...(Args) == N, "Incorrect number of subkeys.");
-    static_assert(
-        (IsValidSubkey<folly::remove_cvref_t<Args>> && ...),
-        "Arguments must be strings or integers");
+  std::pair<std::string_view, std::reference_wrapper<CachedType>>
+  getFormattedKeyWithExtra(Args&&... subkeys) {
+    auto& v = getFormattedEntry(std::forward<Args>(subkeys)...);
+    return {*v.key, v.cached};
+  }
 
-    auto decay = folly::overload(decay_<int64_t>{}, decay_<std::string_view>{});
-    // calling outline folly::get_default would be a small perf hit
-    auto const& map = *localMap_; // so call map-find inline to avoid
-    auto it = map.find(std::tuple{decay(subkeys)...});
-    return FOLLY_LIKELY(it != map.end())
-        ? *it->second
-        : getFormattedKeySlow(std::forward<Args>(subkeys)...);
+  template <typename... Args>
+  const std::string& getFormattedKey(Args&&... subkeys) {
+    return *getFormattedEntry(std::forward<Args>(subkeys)...).key;
   }
 
   template <typename... Args>
@@ -613,6 +630,22 @@ class FormattedKeyHolder {
   }
 
  private:
+  template <typename... Args>
+  ValueType& getFormattedEntry(Args&&... subkeys) {
+    static_assert(sizeof...(Args) == N, "Incorrect number of subkeys.");
+    static_assert(
+        (IsValidSubkey<folly::remove_cvref_t<Args>> && ...),
+        "Arguments must be strings or integers");
+
+    auto decay = folly::overload(decay_<int64_t>{}, decay_<std::string_view>{});
+    // calling outline folly::get_default would be a small perf hit
+    auto& map = *localMap_; // so call map-find inline to avoid
+    auto it = map.find(std::tuple{decay(subkeys)...});
+    return FOLLY_LIKELY(it != map.end())
+        ? it->second
+        : getFormattedKeySlow(std::forward<Args>(subkeys)...);
+  }
+
   template <typename T>
   struct decay_ {
     FOLLY_ERASE constexpr T operator()(T const& t) const {
@@ -632,11 +665,12 @@ class FormattedKeyHolder {
    * corresponding formatted-key.
    */
   template <typename... Args>
-  const std::string& getFormattedKeySlow(Args&&... subkeys) {
+  ValueType& getFormattedKeySlow(Args&&... subkeys) {
     auto mkvar = folly::overload(
         mkvar_<int64_t>{}, mkvar_<std::string_view, std::string>{});
     auto const& [k, v] = getFormattedKeyGlobal({mkvar(subkeys)...});
-    return *localMap_->emplace(std::cref(k), &v).first->second;
+    return localMap_->emplace(std::cref(k), ValueType{&v, CachedType{}})
+        .first->second;
   }
 
   /**
@@ -684,7 +718,9 @@ class FormattedKeyHolder {
           [](std::string const& v) { return v; });
     }
     auto formattedKey = folly::svformat(keyFormat_, subkeyStrings);
-    prepareKey_(formattedKey);
+    if (prepareKey_) {
+      prepareKey_(formattedKey);
+    }
 
     auto writePtr = upgradePtr.moveFromUpgradeToWrite();
     iter = writePtr->emplace(subkeyArray, std::move(formattedKey)).first;
@@ -1036,6 +1072,70 @@ struct MinuteOnlyTimeseriesWrapper : public TimeseriesWrapperBase {
   static const ExportedStat& templateExportedStat();
 };
 
+struct SubminuteMinuteTimeseriesWrapper : public TimeseriesWrapperBase {
+ public:
+  template <typename... Args>
+  explicit SubminuteMinuteTimeseriesWrapper(
+      std::string key,
+      const Args&... args)
+      : TimeseriesWrapperBase(std::move(key)) {
+    exportStats(args...);
+  }
+
+ protected:
+  std::shared_ptr<ThreadCachedServiceData::TLTimeseries> getStatSafe(
+      const std::string& key) override {
+    return ThreadCachedServiceData::getStatsThreadLocal()->getTimeseriesSafe(
+        key, (size_t)60, (size_t)1, fb303::kSubminuteMinuteDurations);
+  }
+
+ private:
+  template <typename... Args>
+  void exportStats(const Args&... args) {
+    // Created counters will export with fb303::kSubminuteMinuteDurations
+    // levels.
+    int _[] = {
+        (ServiceData::get()->addStatExportType(
+             key_, args, &templateExportedStat()),
+         0)...};
+    (void)_;
+  }
+
+  static const ExportedStat& templateExportedStat();
+};
+
+struct SubminuteMinuteOnlyTimeseriesWrapper : public TimeseriesWrapperBase {
+ public:
+  template <typename... Args>
+  explicit SubminuteMinuteOnlyTimeseriesWrapper(
+      std::string key,
+      const Args&... args)
+      : TimeseriesWrapperBase(std::move(key)) {
+    exportStats(args...);
+  }
+
+ protected:
+  std::shared_ptr<ThreadCachedServiceData::TLTimeseries> getStatSafe(
+      const std::string& key) override {
+    return ThreadCachedServiceData::getStatsThreadLocal()->getTimeseriesSafe(
+        key, (size_t)60, (size_t)1, fb303::kSubminuteMinuteOnlyDurations);
+  }
+
+ private:
+  template <typename... Args>
+  void exportStats(const Args&... args) {
+    // Created counters will export with fb303::kSubminuteMinuteOnlyDurations
+    // levels.
+    int _[] = {
+        (ServiceData::get()->addStatExportType(
+             key_, args, &templateExportedStat()),
+         0)...};
+    (void)_;
+  }
+
+  static const ExportedStat& templateExportedStat();
+};
+
 class HistogramWrapper {
  public:
   template <typename... Args>
@@ -1095,6 +1195,24 @@ class MinuteOnlyHistogram : public HistogramWrapper {
             MinuteOnlyTimeSeries<CounterType>()) {}
 };
 
+class SubminuteMinuteOnlyHistogram : public HistogramWrapper {
+ public:
+  template <typename... Args>
+  SubminuteMinuteOnlyHistogram(
+      const std::string& key,
+      int64_t bucketWidth,
+      int64_t min,
+      int64_t max,
+      const Args&... args)
+      : HistogramWrapper(
+            key,
+            bucketWidth,
+            min,
+            max,
+            args...,
+            SubminuteMinuteOnlyTimeSeries<CounterType>()) {}
+};
+
 /**
  * DynamicTimeseriesWrapper is similar to TimeseriesWrapper, but instead of
  * having a fixed key, it takes a key format (e.g. "foo.{}") containing one or
@@ -1122,6 +1240,10 @@ class MinuteOnlyHistogram : public HistogramWrapper {
 template <int N> // N is the number of subkeys.
 class DynamicTimeseriesWrapper {
  public:
+  using KeyHolder = internal::FormattedKeyHolder<
+      N,
+      std::shared_ptr<ThreadCachedServiceData::TLTimeseries>>;
+
   DynamicTimeseriesWrapper(
       std::string keyFormat,
       std::vector<ExportType> exportTypes)
@@ -1167,8 +1289,14 @@ class DynamicTimeseriesWrapper {
   //      add(2, "red", 42);
   template <typename... Args>
   void add(int64_t value, Args&&... subkeys) {
-    auto const& key = key_.getFormattedKey(std::forward<Args>(subkeys)...);
-    tcData().addStatValue(key, value);
+    auto key = key_.getFormattedKeyWithExtra(std::forward<Args>(subkeys)...);
+    if (key.second.get() == nullptr) {
+      ThreadCachedServiceData::ThreadLocalStatsMap& tcData =
+          *ThreadCachedServiceData::getStatsThreadLocal();
+      // Cache thread local counter
+      key.second.get() = tcData.getTimeseriesSafe(key.first);
+    }
+    key.second.get()->addValue(value);
   }
 
   // "subkeys" must be a list of exactly N strings or integers, one for each
@@ -1210,7 +1338,7 @@ class DynamicTimeseriesWrapper {
 
   // Returns a copy of the global map.
   // Only for debugging; not designed to be efficient.
-  typename internal::FormattedKeyHolder<N>::GlobalMap getMap() const {
+  typename KeyHolder::GlobalMap getMap() const {
     return key_.getMap();
   }
 
@@ -1239,7 +1367,7 @@ class DynamicTimeseriesWrapper {
     }
   }
 
-  internal::FormattedKeyHolder<N> key_;
+  KeyHolder key_;
   std::vector<ExportType> exportTypes_;
 };
 
@@ -1289,8 +1417,7 @@ class DynamicHistogramWrapper {
   const internal::HistogramSpec spec_;
 };
 
-} // namespace fb303
-} // namespace facebook
+} // namespace facebook::fb303
 
 namespace facebook {
 
