@@ -31,9 +31,15 @@ namespace fb303 {
  */
 
 template <class LockTraits>
-TLStatT<LockTraits>::TLStatT(const Container* stats, folly::StringPiece name)
+TLStatT<LockTraits>::TLStatT(
+    const Container* stats,
+    std::shared_ptr<const std::string> namePtr)
     : link_{typename detail::TLStatLinkPtr<LockTraits>::FromOther{}, stats->link_},
-      name_(name.str()) {}
+      name_(std::move(namePtr)) {}
+
+template <class LockTraits>
+TLStatT<LockTraits>::TLStatT(const Container* stats, folly::StringPiece name)
+    : TLStatT(stats, std::make_shared<std::string>(name.str())) {}
 
 template <class LockTraits>
 TLStatT<LockTraits>::~TLStatT() {
@@ -170,7 +176,7 @@ auto TLStatT<LockTraits>::withContainerChecked(const char* errorMsg, Fn&& fn) {
   auto guard = link_->lock();
   if (!link_->container_) {
     throw std::runtime_error(folly::to<std::string>(
-        name_,
+        name(),
         ": ThreadLocalStats container has already been destroyed while ",
         errorMsg));
   }
@@ -178,6 +184,11 @@ auto TLStatT<LockTraits>::withContainerChecked(const char* errorMsg, Fn&& fn) {
   return fn(*link_->container_);
 }
 
+template <typename LockTraits>
+bool TLStatT<LockTraits>::shouldUpdateGlobalStatsOnRead() const {
+  return link_->shouldUpdateGlobalStatsOnRead();
+}
+
 /*
  * TLTimeseriesT
  */
@@ -247,14 +258,14 @@ void TLTimeseriesT<LockTraits>::exportStat(fb303::ExportType exportType) {
       globalStat_,
       this->name(),
       exportType,
-      detail::shouldUpdateGlobalStatOnRead());
+      this->shouldUpdateGlobalStatsOnRead());
 }
 
 template <class LockTraits>
 void TLTimeseriesT<LockTraits>::aggregate(std::chrono::seconds now) {
   auto [currentCount, currentSum] = value_.reset();
   if (currentCount == 0) {
-    if (!detail::shouldUpdateGlobalStatOnRead()) {
+    if (!this->shouldUpdateGlobalStatsOnRead()) {
       // We must call update() here so that the stat decays properly
       // if no samples were added.
       auto lockedStatPtr = globalStat_.lock();
@@ -270,7 +281,7 @@ void TLTimeseriesT<LockTraits>::aggregate(std::chrono::seconds now) {
   auto lockedStatPtr = globalStat_.lock();
   lockedStatPtr->addValueAggregated(now, currentSum, currentCount);
 
-  if (!detail::shouldUpdateGlobalStatOnRead()) {
+  if (!this->shouldUpdateGlobalStatsOnRead()) {
     // We must call update() after aggregation at least once
     // so that subsequent reads see it.
     lockedStatPtr->update(now.count());
@@ -493,8 +504,11 @@ void TLCounterT<LockTraits>::aggregate() {
  */
 
 template <class LockTraits>
-ThreadLocalStatsT<LockTraits>::ThreadLocalStatsT(ServiceData* serviceData)
+ThreadLocalStatsT<LockTraits>::ThreadLocalStatsT(
+    ServiceData* serviceData,
+    bool updateGlobalStatsOnRead)
     : serviceData_{serviceData ? serviceData : fbData.ptr()},
+      updateGlobalStatsOnRead_{updateGlobalStatsOnRead},
       link_{new detail::TLStatLink<LockTraits>{this}} {}
 
 template <class LockTraits>
