@@ -26,13 +26,13 @@
 #include <fb303/ExportedStatMapImpl.h>
 #include <fb303/ServiceData.h>
 
+#include <atomic>
 #include <chrono>
 #include <mutex>
 #include <shared_mutex>
 #include <string>
 
-namespace facebook {
-namespace fb303 {
+namespace facebook::fb303 {
 
 template <class LockTraits>
 class TLStatT;
@@ -187,15 +187,17 @@ bool shouldUpdateGlobalStatOnRead();
 template <class LockTraits>
 class ThreadLocalStatsT {
  public:
-  typedef TLCounterT<LockTraits> TLCounter;
-  typedef TLHistogramT<LockTraits> TLHistogram;
-  typedef TLTimeseriesT<LockTraits> TLTimeseries;
+  using TLCounter = TLCounterT<LockTraits>;
+  using TLHistogram = TLHistogramT<LockTraits>;
+  using TLTimeseries = TLTimeseriesT<LockTraits>;
 
   /**
    * Create a new ThreadLocalStats container. Per default (NULL),
    * serviceData will be initialized to facebook::fb303::fbData
    */
-  explicit ThreadLocalStatsT(ServiceData* serviceData = nullptr);
+  explicit ThreadLocalStatsT(
+      ServiceData* serviceData = nullptr,
+      bool updateGlobalStatsOnRead = detail::shouldUpdateGlobalStatOnRead());
 
   virtual ~ThreadLocalStatsT();
 
@@ -282,6 +284,9 @@ class ThreadLocalStatsT {
   // from multiple threads.
   ServiceData* const serviceData_;
 
+  // See detail::shouldUpdateGlobalStatsOnRead().
+  bool updateGlobalStatsOnRead_;
+
   /**
    * ThreadLocalStats and every TLStat in tlStats_ has a pointer to
    * the TLStatLink.  On destruction, ThreadLocalStats clears
@@ -298,8 +303,8 @@ class ThreadLocalStatsT {
    */
   folly::F14FastSet<TLStat*> tlStats_;
 
-  template <typename T>
-  friend class TLStatT;
+  friend class TLStatT<LockTraits>;
+  friend class detail::TLStatLink<LockTraits>;
 };
 
 /**
@@ -313,9 +318,15 @@ class TLStatT {
   using Container = ThreadLocalStatsT<LockTraits>;
 
   TLStatT(const Container* stats, folly::StringPiece name);
+  TLStatT(const Container* stats, std::shared_ptr<const std::string> namePtr);
   virtual ~TLStatT();
 
   const std::string& name() const {
+    static const std::string kEmpty;
+    return name_ ? *name_ : kEmpty;
+  }
+
+  std::shared_ptr<const std::string> namePtr() const {
     return name_;
   }
 
@@ -400,6 +411,12 @@ class TLStatT {
   template <typename Fn>
   auto withContainerChecked(const char* errorMsg, Fn&& fn);
 
+  /**
+   * Check if we should call update on the read path.
+   * See detail::shouldUpdateGlobalStatsOnRead() for details.
+   */
+  bool shouldUpdateGlobalStatsOnRead() const;
+
  private:
   /**
    * Explicitly deleted move andy copy constructors.
@@ -439,7 +456,7 @@ class TLStatT {
   void unlink();
 
   detail::TLStatLinkPtr<LockTraits> link_;
-  std::string name_;
+  std::shared_ptr<const std::string> name_;
 
   /**
    * Synchronizes access to this TLStat's value. This class used to
@@ -516,6 +533,19 @@ class TLTimeseriesT : public TLStatT<LockTraits> {
    */
   TLTimeseriesT& operator=(TLTimeseriesT&& other) noexcept(false);
 
+  /**
+   * Copy the TLTimeseriesT object,
+   * linking it with a different ThreadLocalStatsT.
+   */
+  explicit TLTimeseriesT(
+      ThreadLocalStatsT<LockTraits>* stats,
+      const TLTimeseriesT& other)
+      : TLStatT<LockTraits>(stats, other.namePtr()),
+        globalStat_(other.globalStat_) {
+    DCHECK(!globalStat_.isNull());
+    this->postInit();
+  }
+
   /**
    * Add a new data point
    */
@@ -791,7 +821,8 @@ class TLStatLink {
   using Lock = typename LockTraits::RegistryLock;
 
   explicit TLStatLink(Container* container)
-      : container_{container}, refCount_{1} {}
+      : updateGlobalStatsOnRead_{container->updateGlobalStatsOnRead_},
+        container_{container} {}
 
   TLStatLink(const TLStatLink&) = delete;
   TLStatLink(TLStatLink&&) = delete;
@@ -800,18 +831,13 @@ class TLStatLink {
   TLStatLink& operator=(TLStatLink&&) = delete;
 
   void incRef() {
-    std::unique_lock guard{mutex_};
-    ++refCount_;
+    refCount_.fetch_add(1, std::memory_order_relaxed);
   }
 
   void decRef() {
-    size_t after;
-    {
-      std::unique_lock guard{mutex_};
-      DCHECK_GT(refCount_, 0u);
-      after = --refCount_;
-    }
-    if (after == 0) {
+    auto before = refCount_.fetch_sub(1, std::memory_order_acq_rel);
+    DCHECK_GT(before, 0); // Or else we've underflowed.
+    if (before == 1) {
       delete this;
     }
   }
@@ -824,21 +850,23 @@ class TLStatLink {
     return std::unique_lock{mutex_};
   }
 
+  bool shouldUpdateGlobalStatsOnRead() const {
+    return updateGlobalStatsOnRead_;
+  }
+
  private:
-  /**
-   * Protects refcount_, container_, and container_->tlStats_.
-   */
+  // Caches the corresponding field in container_ so that it's
+  // still accessible after the container has been destroyed.
+  const bool updateGlobalStatsOnRead_;
+
+  // Protects container_ and container_->tlStats_.
   Lock mutex_;
 
   // If container_ is non-null, then the pointee Container is guaranteed to be
   // alive. ThreadLocalStats's destructor zeroes this pointer.
-  Container* container_{nullptr};
+  Container* container_ = nullptr;
 
-  // TODO: It's slightly inefficient to keep the refcount under a
-  // mutex. It is not updated simultaneously with container_ or
-  // tlStats_, so it could be replaced with std::atomic<size_t> or
-  // size_t depending on LockTraits.
-  size_t refCount_{0};
+  std::atomic<size_t> refCount_ = 1;
 
   friend class ThreadLocalStatsT<LockTraits>;
   friend class TLStatT<LockTraits>;
@@ -937,7 +965,6 @@ class TLStatLinkPtr {
 };
 } // namespace detail
 
-} // namespace fb303
-} // namespace facebook
+} // namespace facebook::fb303
 
 #include <fb303/ThreadLocalStats-inl.h>
